/*
    Game Genie Decoder Program v 3.2
    Copyright (C) 2001 John David Ratliff
    Questions or Comments: 
    	E-Mail: jdratlif@cs.indiana.edu
    	URL:	http://www.technoplaza.net 
    	
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 5/28/01; 4:46:10 AM

#include "ggdecode.h"

#ifdef _GBGG_SUPPORT_

#include <string.h>
#include <dialogs.h>
#include <stdio.h>
#include <alloc.h>

short int isValidHex(const char digit) {
	if ((digit >= '0' && digit <= '9') || 
	    (digit >= 'A' && digit <= 'F') ||
	    (digit >= 'a' && digit <= 'f')) {
	    return 1;
	}
	
	return 0;
}

short int isValidGBGG(unsigned char *ggcode, short int *length) {
	short int loop;

	if (*length == 11) {
		if (ggcode[7] != '-' && ggcode[7] != 173) {
			return -1;
		} else {
			for (loop = 7; loop < *length; loop++) {
				ggcode[loop] = ggcode[loop+1];
			}
			
			ggcode[--(*length)] = 0;
		}
		
		if (ggcode[3] != '-' && ggcode[3] != 173) {
			return -1;
		} else {
			for (loop = 3; loop < *length; loop++) {
				ggcode[loop] = ggcode[loop+1];
			}
			
			ggcode[--(*length)] = 0;
		}
	} else if (*length == 7) {
		if (ggcode[3] != '-' && ggcode[3] != 173) {
			return -1;
		} else {
			for (loop = 3; loop < *length; loop++) {
				ggcode[loop] = ggcode[loop+1];
			}
			
			ggcode[--(*length)] = 0;			
		}
	} else if (*length != 6 && *length != 9) {
		return -1;
	}
	
	for (loop = 0; loop < *length; loop++) {
		if (isValidHex(ggcode[loop]) == 0) {
			return -2;
		}
	}
	
	return 1;
}

void decodeGBGG(const char *ggcode, char *addystr, char *valuestr, char *comparestr, short int length) {
	char str[13];
	
	// make a backup of the ggcode string
	strcpy(str,ggcode);
	
	str[2] = 0;
	strcat(valuestr,str);
	str[2] = ggcode[2];
	
	str[6] = 0;
	strcat(addystr,str+2);
	str[6] = ggcode[6];

	if (length == 9) {
		str[7] = str[8];
		str[8] = 0;
		
		strcat(comparestr,str+6);
	}
}

void convertGBGG(char *ggcode) {
	unsigned short int address = 0, value = 0, compare = 0;
	short int length = strlen(ggcode), result;
	HANDLE dlg = H_NULL;
	char str[30],saveCode[13],addystr[5],valuestr[3],comparestr[3];
	
	// preserve original code
	strcpy(saveCode,ggcode);
	
	// erase the bit strings
	addystr[0] = 0;
	valuestr[0] = 0;
	comparestr[0] = 0;
	
	result = isValidGBGG(ggcode,&length);
	
	if (result == -1) {
		DlgMessage((char *)errors[INVALID_LENGTH],"Game Boy/Game Gear codes must be 6 or 9 characters!",BT_OK,BT_NONE);
	} else if (result == -2) {
		DlgMessage((char *)errors[INVALID_CODE],"Only 0123456789ABCDEF are valid!",BT_OK,BT_NONE);
	} else {
		if ((dlg = DialogNewSimple(GBGG_WIDTH,GBGG_HEIGHT)) != H_NULL) {
			decodeGBGG(ggcode,addystr,valuestr,comparestr,length);
			
			value = (unsigned short int)toDecimal(valuestr,16);
			
			address = (unsigned short int)toDecimal(addystr,16);
			address = (address >> 4) | (~(address & 0x000F) << 12);
			
			if (length == 9) {
				compare = (unsigned short int)toDecimal(comparestr,16);
				compare = (((compare >> 2) | (compare << 6)) ^ 0xBA) & 0x00FF;
			}	
		
			DialogAddTitle(dlg,titles[GBGG],BT_OK,BT_NONE);
			
			sprintf(str,ggstr,saveCode);
			DialogAddText(dlg,5,15,str);
			
			if (length == 6) {
				sprintf(str,"Value: %02hXh",value);
			} else if (length == 9) {
				sprintf(str,"Value: %02hXh, Compare Value: %02hXh",value,compare);
			}
			
			DialogAddText(dlg,5,25,str);
			
			sprintf(str,"CPU Address: %04hXh",address);
			DialogAddText(dlg,5,35,str);
			
			DialogDo(dlg,CENTER,CENTER,NULL,NULL);
			
			HeapFree(dlg);
		} else {
			DlgMessage((char *)errors[MEMORY_ERROR],(char *)errors[NO_FREE_MEMORY],BT_OK,BT_NONE);
		}
	}
}

#endif
