/*
    Game Genie Decoder Program v 3.2
    Copyright (C) 2001 John David Ratliff
    Questions or Comments: 
    	E-Mail: jdratlif@cs.indiana.edu
    	URL:	http://www.technoplaza.net 
    	
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 5/27/01; 8:22:28 PM

#include "ggdecode.h"

#ifdef _NES_SUPPORT_

#include <string.h>
#include <dialogs.h>
#include <stdio.h>
#include <alloc.h>

char decodeNEStoHex(const char digit) {
	char decoded;
	
	switch (digit) {
		case 'A': decoded = '0'; break;
		case 'P': decoded = '1'; break;
		case 'Z': decoded = '2'; break;
		case 'L': decoded = '3'; break;
		case 'G': decoded = '4'; break;
		case 'I': decoded = '5'; break;
		case 'T': decoded = '6'; break;
		case 'Y': decoded = '7'; break;
		case 'E': decoded = '8'; break;
		case 'O': decoded = '9'; break;
		case 'X': decoded = 'A'; break;
		case 'U': decoded = 'B'; break;
		case 'K': decoded = 'C'; break;
		case 'S': decoded = 'D'; break;
		case 'V': decoded = 'E'; break;
		case 'N': decoded = 'F'; break;
		default:
			decoded = 0;
			break;
	}
	
	return decoded;
}

short int isValidNES(char *ggcode, short int length) {
	short int loop;
	
	for (loop = 0; loop < length; loop++) {
		if ((ggcode[loop] = decodeNEStoHex(ggcode[loop])) == 0) {
			return -1;
		}
	}
	
	return 1;
}

void decodeNES6(unsigned long int cipher, unsigned short int *value, unsigned short int *address) {
	unsigned long int segment, decrypted = 0;
	
	segment = (cipher & 0x00800000);
	decrypted |= segment;
	
	segment = (cipher & 0x00070000) << 4;
	decrypted |= segment;
	
	segment = (cipher & 0x00000008) << 16;
	decrypted |= segment;
	
	segment = (cipher & 0x00700000) >> 4;
	decrypted |= segment;
	
	segment = (cipher & 0x00000780) << 5;
	decrypted |= segment;
	
	segment = (cipher & 0x00000007) << 9;
	decrypted |= segment;
	
	segment = (cipher & 0x00080000) >> 11;
	decrypted |= segment;
	
	segment = (cipher & 0x00007000) >> 7;
	decrypted |= segment;
	
	segment = (cipher & 0x00000800) >> 7;
	decrypted |= segment;
	
	segment = (cipher & 0x00000070) >> 3;
	decrypted |= segment;
	
	*value = (unsigned short int)(decrypted >> 16);	
	*address = (unsigned short int)((decrypted & 0x0000FFFF) >> 1);
}

void decodeNES8(unsigned long int cipher, unsigned short int *value, unsigned short int *address,
		unsigned short int *compare) {
	unsigned long int segment, decrypted = 0;
	
	segment = (cipher & 0x80000000);
	decrypted |= segment;
	
	segment = (cipher & 0x07000000) << 4;
	decrypted |= segment;
	
	segment = (cipher & 0x00000008) << 24;
	decrypted |= segment;
	
	segment = (cipher & 0x70000000) >> 4;
	decrypted |= segment;
	
	segment = (cipher & 0x00078000) << 5;
	decrypted |= segment;
	
	segment = (cipher & 0x00000700) << 9;
	decrypted |= segment;
	
	segment = (cipher & 0x08000000) >> 11;
	decrypted |= segment;
	
	segment = (cipher & 0x00700000) >> 7;
	decrypted |= segment;
	
	segment = (cipher & 0x00080000) >> 7;
	decrypted |= segment;
	
	segment = (cipher & 0x00007000) >> 3;
	decrypted |= segment;
	
	segment = (cipher & 0x00000080);
	decrypted |= segment;
	
	segment = (cipher & 0x00000007) << 4;
	decrypted |= segment;
	
	segment = (cipher & 0x00000070) >> 4;
	decrypted |= segment;
	
	segment = (cipher & 0x00000800) >> 11;
	decrypted |= segment;
	
	*value = (unsigned short int)(decrypted >> 24);
	*address = (unsigned short int)((decrypted & 0x00FFFF00) >> 9);
	*compare = (unsigned short int)(decrypted & 0x000000FF);
}

void convertNES(char *ggcode) {
	HANDLE dlg = H_NULL;
	short int length = strlen(ggcode);
	unsigned short int value, address, compare = -1;
	unsigned long int cipher;
	char saveCode[13],str[30];
	
	// save the old code...
	strcpy(saveCode,ggcode);
	
	// make sure length is valid
	if (length != 6 && length != 8) {
		DlgMessage((char *)errors[INVALID_LENGTH],"NES codes must be 6 or 8 characters!",BT_OK,BT_NONE);
		return;
	}
	
	// check for code validity and convert code digits to hex
	if (isValidNES(ggcode,length) == 1) {
		// convert to binary
		cipher = toDecimal(ggcode,16);
		
		if (length == 6) {
			decodeNES6(cipher,&value,&address);
		} else {
			decodeNES8(cipher,&value,&address,&compare);
		}
		
		if ((dlg = DialogNewSimple(NES_WIDTH,NES_HEIGHT)) == H_NULL) {
			DlgMessage((char *)errors[MEMORY_ERROR],(char *)errors[NO_FREE_MEMORY],BT_OK,BT_NONE);
		} else {
			DialogAddTitle(dlg,titles[NES],BT_OK,BT_NONE);
			
			sprintf(str,ggstr,saveCode);
			DialogAddText(dlg,5,15,str);
			
			if (compare != 0xFFFF) {
				sprintf(str,"Value: %02hXh, Compare: %02hXh",value,compare);
			} else {
				sprintf(str,"Value: %02hXh",value);
			}
			
			DialogAddText(dlg,5,25,str);
			
			sprintf(str,"Code Address: %04hXh",address);
			DialogAddText(dlg,5,35,str);
			
			sprintf(str,"CPU Address: %04hXh",address+0x8000);
			DialogAddText(dlg,5,45,str);
			
			sprintf(str,"ROM Address: %04hXh",address+0x0010);
			DialogAddText(dlg,5,55,str);
			
			DialogDo(dlg,CENTER,CENTER,NULL,NULL);
			
			HeapFree(dlg);
		}		
	} else {
		DlgMessage((char *)errors[INVALID_CODE],"Only APZLGITYEOXUKSVN are valid!",BT_OK,BT_NONE);
	}
}

#endif
